@@ -7,12 +7,13 @@ from ...._proto_http_async import HttpProto
 from ...._proto_socks4_async import Socks4Proto
 from ...._proto_socks5_async import Socks5Proto
 
-from ._stream import SocketStream
+from ...._resolver_async_aio import Resolver
+from ._stream import AsyncioSocketStream
 
 DEFAULT_TIMEOUT = 60
 
 
-class BaseProxy:
+class AsyncioProxy:
     def __init__(self, proxy_host, proxy_port, proxy_ssl=None,
                  loop: asyncio.AbstractEventLoop = None):
 
@@ -29,11 +30,13 @@ class BaseProxy:
         self._dest_port = None
         self._timeout = None
 
-        self._stream = SocketStream(loop=loop)
+        self._stream = AsyncioSocketStream(loop=loop)
+        self._resolver = Resolver(loop=loop)
+
         self._in_chain = False
 
     async def connect(self, dest_host, dest_port,
-                      timeout=DEFAULT_TIMEOUT) -> SocketStream:
+                      timeout=DEFAULT_TIMEOUT) -> AsyncioSocketStream:
         self._dest_host = dest_host
         self._dest_port = dest_port
         self._timeout = timeout
@@ -71,12 +74,12 @@ class BaseProxy:
                 raise
 
             try:
-                await self.negotiate()
+                await self._negotiate()
             except Exception:
                 await self._stream.close()
                 raise
 
-    async def negotiate(self):
+    async def _negotiate(self):
         raise NotImplementedError()  # pragma: no cover
 
     @property
@@ -88,7 +91,7 @@ class BaseProxy:
         return self._proxy_port
 
 
-class Socks5Proxy(BaseProxy):
+class Socks5Proxy(AsyncioProxy):
     def __init__(self, proxy_host, proxy_port,
                  username=None, password=None, rdns=None,
                  proxy_ssl=None,
@@ -103,9 +106,10 @@ class Socks5Proxy(BaseProxy):
         self._password = password
         self._rdns = rdns
 
-    async def negotiate(self):
+    async def _negotiate(self):
         proto = Socks5Proto(
-            stream=self._stream,  # noqa
+            stream=self._stream,
+            resolver=self._resolver,
             dest_host=self._dest_host,
             dest_port=self._dest_port,
             username=self._username,
@@ -115,7 +119,7 @@ class Socks5Proxy(BaseProxy):
         await proto.negotiate()
 
 
-class Socks4Proxy(BaseProxy):
+class Socks4Proxy(AsyncioProxy):
     def __init__(self, proxy_host, proxy_port,
                  user_id=None, rdns=None, proxy_ssl=None,
                  loop: asyncio.AbstractEventLoop = None):
@@ -128,9 +132,10 @@ class Socks4Proxy(BaseProxy):
         self._user_id = user_id
         self._rdns = rdns
 
-    async def negotiate(self):
+    async def _negotiate(self):
         proto = Socks4Proto(
-            stream=self._stream,  # noqa
+            stream=self._stream,
+            resolver=self._resolver,
             dest_host=self._dest_host,
             dest_port=self._dest_port,
             user_id=self._user_id,
@@ -139,7 +144,7 @@ class Socks4Proxy(BaseProxy):
         await proto.negotiate()
 
 
-class HttpProxy(BaseProxy):
+class HttpProxy(AsyncioProxy):
     def __init__(self, proxy_host, proxy_port,
                  username=None, password=None, proxy_ssl=None,
                  loop: asyncio.AbstractEventLoop = None):
@@ -152,7 +157,7 @@ class HttpProxy(BaseProxy):
         self._username = username
         self._password = password
 
-    async def negotiate(self):
+    async def _negotiate(self):
         proto = HttpProto(
             stream=self._stream,  # noqa
             dest_host=self._dest_host,
